/*
 * Button.h
 *
 * The data structure declarations and functional prototypes for the
 * Button class of interface tools.
 *
 * Button as defined here is an object that has 1 or more states (numbered
 * from 0 to n-1 for consideration). For each state as defined, there
 * exists a list of callback-like routines, general display and color
 * information, and visual information. There are some data items that
 * exist in only one instance for each button, such as size and position
 * information.
 *
 */

#ifndef _XF_BUTTON_H
#define _XF_BUTTON_H

/* Information necessary to define a state: callbacks and visual data */
struct ButtonState
{
  struct CallBackList* CallBacks; /* List of call backs for specific state */
  struct VisualInfo* Visuals;   /* Visual elements for button */
};

/* For typing ease */
typedef struct _Button* Button;

/* The underlying structure, defining the definitive button */
struct _Button
{
  Display* display;             /* Display that button is on */
  Window window;                /* Window ID of the button */
  Window parent;                /* Parent's Window ID */
  int active;                   /* Is this button active? */
  int width, height;            /* sizing of the button */
  int x, y;                     /* X and Y position of button wrt parent */
  int border_width;             /* width in pixels of button's border */
  unsigned long border_color;   /* color for window's border */
  char name[256];               /* Name used to identify the button */
  char* ext;                    /* An externally-visible piece of data */
  int *member;                  /* A pseudo-class identifier */
  int state;                    /* Current state of the button */
  int maxstate;                 /* Number of states */
  int noAutoExpose;             /* Don't Handle Exposure Events automatically */
  struct ButtonState** States;  /* Dynamically allocated state data */
  CallBack exposeCallback;      /* Callback action on expose events */
  CallBack updateCallback;      /* Callback action forced, to update */
  Button nextButton;            /* Pointer for linked list */
};

/* The support routines for general manipulation */
Button XfCreateButton();
int XfDestroyButton();
int XfActivateButtonState();
int XfDeactivateButton();
int XfAddButtonVisual();
int XfAddButtonCallback();
int XfDelButtonCallback();
int XfButtonPush();
int XfMoveButton();
Button XfGetButton();
Button XfEventButton();

#define XfActivateButton(B, mask)       XfActivateButtonState(B, 0, mask)

/* One application that is too easy to waste code on */
#define XfButtonState(B) (B)->state

/* The global list of buttons */
extern Button ButtonList;

#endif /* _XF_BUTTON_H */
